package info.textgrid.middleware.test;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;
import org.json.JSONObject;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.DublinCoreBuilder;
import info.textgrid.middleware.IdentifierListDelivererDC;
import info.textgrid.middleware.IdentifierListDelivererDatacite;
import info.textgrid.middleware.IdentifierListDelivererIdiom;
import info.textgrid.middleware.IdiomImages;
import info.textgrid.middleware.MetadataFormatListDelivererInterface;
import info.textgrid.middleware.MetadataFormatListDelivererTG;
import info.textgrid.middleware.OaipmhConstants;
import info.textgrid.middleware.OaipmhElasticSearchClient;
import info.textgrid.middleware.OaipmhImpl;
import info.textgrid.middleware.OaipmhTGConstants;
import info.textgrid.middleware.OaipmhUtilities;
import info.textgrid.middleware.RecordDelivererDC;
import info.textgrid.middleware.RecordDelivererDatacite;
import info.textgrid.middleware.RecordDelivererIdiom;
import info.textgrid.middleware.RecordListDelivererDC;
import info.textgrid.middleware.RecordListDelivererDatacite;
import info.textgrid.middleware.RecordListDelivererIdiom;
import info.textgrid.middleware.RepIdentification;
import info.textgrid.middleware.SetListDeliverer;
import info.textgrid.middleware.common.TextGridMimetypes;
import info.textgrid.middleware.oaipmh.DeletedRecordType;
import info.textgrid.middleware.oaipmh.GranularityType;

/**
 * FIXME: Add documentation how to use this class, use SSH tunnel?
 * 
 * @author Maximilian Brodhun, SUB Göttingen
 * @author Stefan E. Funk, SUB Göttingen
 */
@Ignore
public class TestTGOaipmhLocally {

  // **
  // STATICS
  // **

  public static OaipmhElasticSearchClient oaiEsClient;

  private static RecordDelivererDC record;
  private static RecordDelivererIdiom recordIDIOM;
  private static RecordDelivererDatacite recordDATACITE;
  private static RecordListDelivererDatacite recordListDATACITE;
  private static IdentifierListDelivererDatacite identifierListDATACITE;
  private static RecordListDelivererDC recordList;
  private static RecordListDelivererIdiom recordListIDIOM;
  private static IdentifierListDelivererDC identifierList =
      new IdentifierListDelivererDC(true, false);
  private static IdentifierListDelivererIdiom identifierListIDIOM =
      new IdentifierListDelivererIdiom(true, false);
  private static SetListDeliverer setListTextGrid;
  private static IdiomImages idiomImages;

  // **
  // CLASS
  // **

  private DeletedRecordType deletedRecordType = DeletedRecordType.NO;
  private GranularityType granularityType = GranularityType.YYYY_MM_DD_THH_MM_SS_Z;
  private RepIdentification rep =
      new RepIdentification("TextGrid-Repository", "www.textgridrep.de", this.deletedRecordType,
          "2011-06-11T02:32:40Z", this.granularityType, "2.0", "textgrid-support@gwdg.de");
  private MetadataFormatListDelivererInterface metadataFormatList =
      new MetadataFormatListDelivererTG();
  private OaipmhImpl request = new OaipmhImpl(this.rep,
      TestTGOaipmhLocally.record,
      TestTGOaipmhLocally.recordIDIOM,
      TestTGOaipmhLocally.recordDATACITE,
      TestTGOaipmhLocally.recordList,
      TestTGOaipmhLocally.recordListIDIOM,
      TestTGOaipmhLocally.recordListDATACITE,
      this.metadataFormatList,
      TestTGOaipmhLocally.setListTextGrid,
      TestTGOaipmhLocally.identifierList,
      TestTGOaipmhLocally.identifierListIDIOM,
      TestTGOaipmhLocally.identifierListDATACITE
  // TestTGOaipmhLocally.idiomImages
  );

  OaipmhUtilities settings = new OaipmhUtilities();

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUp() throws Exception {

    int[] ports = new int[] {9202};
    oaiEsClient = new OaipmhElasticSearchClient("localhost", ports, 100);
    oaiEsClient.setEsIndex("textgrid-public");
    oaiEsClient.setEsType("metadata");

    record = new RecordDelivererDC(true, false);
    record.setOaiEsClient(oaiEsClient);
    record.setWorkFields(OaipmhTGConstants.TGWorkFields);

    recordIDIOM = new RecordDelivererIdiom(true, false);
    recordIDIOM.setOaiEsClient(oaiEsClient);

    recordDATACITE = new RecordDelivererDatacite(true, false);
    recordDATACITE.setOaiEsClient(oaiEsClient);

    recordListDATACITE = new RecordListDelivererDatacite(true, false);
    recordListDATACITE.setOaiEsClient(oaiEsClient);

    identifierListIDIOM = new IdentifierListDelivererIdiom(true, false);
    recordList = new RecordListDelivererDC(true, false);
    recordList.setFields(OaipmhTGConstants.TGFields);
    recordList.setWorkFields(OaipmhTGConstants.TGWorkFields);
    recordList.setOaiEsClient(oaiEsClient);
    recordList.setSearchResponseSize(100);

    recordListIDIOM = new RecordListDelivererIdiom(true, false);
    recordListIDIOM.setOaiEsClient(oaiEsClient);

    identifierList = new IdentifierListDelivererDC(true, false);
    identifierList.setOaiEsClient(oaiEsClient);
    identifierList.setIdentifierListFields(OaipmhTGConstants.IDENTIFIER_LIST_FIELDS);

    setListTextGrid = new SetListDeliverer(true, false);
  }


  /**
   * 
   */
  @Test
  public void testGetArrayFromESIndex() {

    String jsonAsString =
        "{\"administrativeMetadata\":{\"dcterms:identifier\":\"hdl:21.T11991/0000-001B-4C02-C\",\"dcterms:modified\":\"2021-06-08T11:03:37.448\"},\"descriptiveMetadata\":{\"dc:creator\":\"fu\",\"dc:format\":\"text/vnd.dariah.dhrep.collection+turtle\",\"dc:rights\":\"free\",\"dc:title\":\"First ES6 Tests for DH-rep using Publikator\",\"dc:identifier\":[\"hdl:21.T11991/0000-001B-4C02-C\",\"doi:10.20375/0000-001B-4C02-C\"]}}";

    JSONObject json = new JSONObject(jsonAsString);
    // System.out.println(json);
    DublinCoreBuilder result = new DublinCoreBuilder();
    List<String> identifier = new ArrayList<String>();
    String[] fieldsForIdentifier = {"descriptiveMetadata.dc:identifier"};
    System.out.println(OaipmhUtilities.fieldLoader(json, OaipmhConstants.TEST_DARIAH_IDENTIFIER));
    identifier = OaipmhUtilities.fieldLoader(json, fieldsForIdentifier);
    for (String id : identifier) {
      System.out.println(id);
    }

    result.setIdentifier(identifier);
  }

  /**
   * 
   */
  @Test
  public void testGetArrayFromESIndexForListRecords() {

    String jsonAsString = "{\r\n"
        + "        \"administrativeMetadata\": {\r\n"
        + "            \"dcterms:created\": \"2021-06-16T17:15:48.689\",\r\n"
        + "            \"dcterms:creator\": \"StefanFunk@dariah.eu\",\r\n"
        + "            \"dcterms:extent\": 154064,\r\n"
        + "            \"dcterms:identifier\": \"hdl:21.T11991/0000-001B-4D41-4\",\r\n"
        + "            \"dcterms:modified\": \"2021-06-16T17:15:48.689\"\r\n"
        + "        },\r\n"
        + "        \"descriptiveMetadata\": {\r\n"
        + "            \"dc:creator\": \"fu\",\r\n"
        + "            \"dc:description\": \"just some ictures\",\r\n"
        + "            \"dc:format\": \"image/jpeg\",\r\n"
        + "            \"dc:identifier\": [\r\n"
        + "                \"hdl:21.T11991/0000-001B-4D41-4\",\r\n"
        + "                \"doi:10.20375/0000-001B-4D41-4\"\r\n"
        + "            ],\r\n"
        + "            \"dc:relation\": \"hdl:21.T11991/0000-001B-4D28-1\",\r\n"
        + "            \"dc:rights\": \"free\",\r\n"
        + "            \"dc:title\": \"2017-07-12-Waffel copy 2.jpeg\"\r\n"
        + "        },\r\n"
        + "        \"technicalMetadata\": {\r\n"
        + "            \"externalIdentifier\": \"fmt/43\",\r\n"
        + "            \"externalIdentifierType\": \"puid\",\r\n"
        + "            \"format\": \"JPEG File Interchange Format\",\r\n"
        + "            \"mimetype\": \"image/jpeg\",\r\n"
        + "            \"valid\": \"true\",\r\n"
        + "            \"version\": \"1.01\",\r\n"
        + "            \"well-formed\": \"true\"\r\n"
        + "        }\r\n"
        + "    }";

    JSONObject json = new JSONObject(jsonAsString);
    DublinCoreBuilder result = new DublinCoreBuilder();
    List<String> identifier = new ArrayList<String>();
    String[] fieldsForIdentifier = {"descriptiveMetadata.dc:identifier"};
    System.out.println(OaipmhUtilities.fieldLoader(json, OaipmhConstants.TEST_DARIAH_IDENTIFIER));
    identifier = OaipmhUtilities.fieldLoader(json, fieldsForIdentifier);
    for (String id : identifier) {
      System.out.println(id);
    }

    result.setIdentifier(identifier);

    System.out.println(result);
  }

  /**
   * 
   */
  @Test
  @Ignore
  public void testOpenAireIdentifierList() {

    this.settings.setRangeField(OaipmhTGConstants.RANGE_FIELD);
    this.settings.setFormatField(OaipmhTGConstants.FORMAT);
    this.settings.setFormatToFilter(TextGridMimetypes.EDITION);
    this.settings.setSearchResponseSize(100);

    String p =
        this.request.getRequest("ListIdentifiers", "", "oai_datacite", "", "2013", "2014", "");

    System.out.println(p);
  }

  /**
   * @throws DatatypeConfigurationException
   */
  @Test
  public void testDateNow() throws DatatypeConfigurationException {

    System.out.println("---------------Test Datestamp Parsing with now date -----------------");
    XMLGregorianCalendar nowTest = OaipmhUtilities.getXMLGregorianCalendarNow();
    System.out.println(nowTest);
    System.out.println("-------------------------------------------");
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  public void testDateGregorianType() throws ParseException, DatatypeConfigurationException {

    String dateformatbefore = "2012-02-06T20:48:39.614+01:00";
    System.out.println("--------- Test Datestamp Parsing with a given date ---------------");
    System.out.println("Original date: " + dateformatbefore);
    XMLGregorianCalendar testDate = OaipmhUtilities.convertDateFormat(dateformatbefore);
    System.out.println("Date after conversion: " + testDate);
    System.out.println("---------------------------------------------");
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  public void testDateStringType() throws ParseException, DatatypeConfigurationException {

    String dateformatbefore = "2012-02-06T20:48:39.614+01:00";
    System.out.println("------------String Version-------------------");
    System.out.println("Original date: " + dateformatbefore);
    String testDate = OaipmhUtilities.oaiDatestampAsString(dateformatbefore);
    System.out.println("Date after conversion: " + testDate);
    System.out.println("---------------------------------------------");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestIdentify() throws ParseException {

    System.out.println("Test for the verb \"Identify\" with successful response");
    String r = this.request.getRequest("Identify", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestIdentifyVerbError() throws ParseException {

    System.out.println("Test for the verb \"Identify\" with error response (MetadaFormat)");
    String t = this.request.getRequest("Identify", "", "a", "", "", "", "");
    System.out.println(t);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestIdentifyArgumentError() throws ParseException {

    System.out.println("Test for the verb \"Identify\" with error response (Until and Identifier)");
    String z = this.request.getRequest("Identify", "a", "", "", "", "a", "");
    System.out.println(z);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetDateFormat() throws ParseException {

    System.out.println("Test get time format");
    System.out.println(OaipmhUtilities.getFormatOfDate("2019-12-02T15:36:13Z"));
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordTextGrid() throws ParseException {

    record.setContributor(OaipmhTGConstants.CONTRIBUTOR_LIST);
    record.setCoverage(OaipmhTGConstants.COVERAGE_LIST);
    record.setCreator(OaipmhTGConstants.CREATOR_LIST);
    record.setDates(OaipmhTGConstants.DATE_LIST);
    record.setDescriptions(OaipmhTGConstants.DESCRIPTION_LIST);
    record.setFormats(OaipmhTGConstants.FORMAT_LIST);
    record.setIdentifiers(OaipmhTGConstants.IDENTIFIER_LIST);
    record.setLanguages(OaipmhTGConstants.LANGUAGE_LIST);
    record.setPublishers(OaipmhTGConstants.PUBLISHER_LIST);
    record.setRelations(OaipmhTGConstants.RELATIONS_LIST);
    // record.setRelationsForWork(TGConstants.RELATIONS_FOR_WORK_LIST);
    record.setRights(OaipmhTGConstants.RIGHTS_LIST);
    record.setSources(OaipmhTGConstants.SOURCE_LIST);
    record.setSubjects(OaipmhTGConstants.SUBJECT_LIST);
    record.setTitles(OaipmhTGConstants.TITLE_LIST);
    record.setTypes(OaipmhTGConstants.TYPE_LIST);
    record.setFields(OaipmhTGConstants.TGFields);
    record.setFormatField(OaipmhTGConstants.FORMAT);
    record.setFormatToFilter(TextGridMimetypes.EDITION);
    record.setDateOfObjectCreation(OaipmhTGConstants.CREATED);
    record.setRelationToFurtherMetadataObject(OaipmhTGConstants.EDITION_ISEDITIONOF);
    record.setRepositoryObjectURIPrefix(OaipmhTGConstants.ITEM_IDENTIFIER_PREFIX);
    record.setIdentifierField("textgridUri");

    System.out.println("Test for the verb \"GetRecord\" with successful response");
    String p = this.request.getRequest("GetRecord", "textgrid:11hp0.0", "oai_dc", "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestListRecordsIDIOMImagesWithResToken() throws ParseException {

    System.out.println("Test for the verb \"ListRecords\" with successful response for IMAGES");
    IdiomImages.getCursorCollector().put("2930c87d-5209-4b14-9d6a-72f10476d947", 129);
    String p = this.request.getRequest("ListRecords", "",
        "", "", "", "", "2930c87d-5209-4b14-9d6a-72f10476d947");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testGetRequestListIdentifiersIDIOMImageswithResToken() throws ParseException {

    System.out.println("Test for the verb \"ListIdentifiers\" with successful response");
    IdiomImages.getCursorCollector().put("8eea2c07-7dc9-4404-a106-c1412b9ce8b4", 129);
    String p = this.request.getRequest("ListIdentifiers", "",
        "", "", "", "", "8eea2c07-7dc9-4404-a106-c1412b9ce8b4");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestGetRecordIDIOM() throws ParseException {

    System.out.println("Test for the verb \"GetRecord\" with successful response");
    String p = this.request.getRequest("GetRecord", "textgrid:3whkp",
        OaipmhConstants.METADATA_IDIOM_PREFIX, "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   * @throws JAXBException
   * @throws DatatypeConfigurationException
   * @throws IOException
   */
  @Test
  @Ignore
  public void testGetRequestGetRecordOpenAire()
      throws ParseException, JAXBException, DatatypeConfigurationException, IOException {

    System.out.println("Test for OpenAire get record");

    System.out.println("-----------------------------------\n");
    // OAIPMHUtilities.marshal(oar.getResource());
    // System.out.println(OAIPMHImpl.getStringFromJAXBOAIElement("GetRecord",
    // oar.getRecord("24gv8.0")));

    // OAIPMHUtilities.marshal(oar.getRecord("24gv8.0"));

    String p = this.request.getRequest("GetRecord", "mk7j.0",
        OaipmhConstants.METADATA_OPENAIRE_PREFIX, "", "", "", "");

    System.out.println(p);
  }

  /**
   * @throws IOException
   */
  @Test
  @Ignore
  public void testLIstRecordsOpenAire() throws IOException {

    recordListDATACITE.setOaiEsClient(oaiEsClient);
    recordListDATACITE.setRangeField(OaipmhTGConstants.RANGE_FIELD);
    recordListDATACITE.setFormatField(OaipmhTGConstants.FORMAT);
    recordListDATACITE.setFormatToFilter(TextGridMimetypes.EDITION);
    recordListDATACITE.setSearchResponseSize(100);

    String p = this.request.getRequest("ListRecords", "", OaipmhConstants.METADATA_OPENAIRE_PREFIX,
        "", "", "", "");

    System.out.println(p);

  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetRequestGetRecordErrorIdentifierMissing() throws ParseException {

    System.out.println("Test for the verb \"GetRecord\" with error response (Identifier Missing)");
    String p = this.request.getRequest("GetRecord", "", "", "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testListIdentifierSetsDC() throws ParseException {

    TestTGOaipmhLocally.identifierList.setFieldForRange(OaipmhTGConstants.RANGE_FIELD);
    TestTGOaipmhLocally.identifierList
        .setIdentifierListFields(OaipmhTGConstants.IDENTIFIER_LIST_FIELDS);
    TestTGOaipmhLocally.identifierList.setDateOfObjectCreation(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.identifierList
        .setRepositoryObjectURIPrefix(OaipmhTGConstants.ITEM_IDENTIFIER_PREFIX);
    TestTGOaipmhLocally.identifierList.setIdentifierField("textgridUri");
    TestTGOaipmhLocally.identifierList.setSearchResponseSize("100");
    System.out.println("Test for the verb \"ListIdentifiers\" with successful response");
    String p =
        this.request.getRequest("ListIdentifiers", "", "oai_dc", "", "2000-02-05", "", "junk");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierWithResumptionTokenDC() throws ParseException {

    TestTGOaipmhLocally.identifierList.setFieldForRange(OaipmhTGConstants.RANGE_FIELD);
    TestTGOaipmhLocally.identifierList
        .setIdentifierListFields(OaipmhTGConstants.IDENTIFIER_LIST_FIELDS);
    TestTGOaipmhLocally.identifierList.setDateOfObjectCreation(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.identifierList
        .setRepositoryObjectURIPrefix(OaipmhTGConstants.ITEM_IDENTIFIER_PREFIX);
    TestTGOaipmhLocally.identifierList.setIdentifierField("textgridUri");
    TestTGOaipmhLocally.identifierList.setSearchResponseSize("100");
    System.out.println("Test for the verb \"ListIdentifiers\" with successful response");
    String p = this.request.getRequest("ListIdentifiers", "", "oai_dc", "", "", "", "");
    System.out.println(p);
    String resToken = "";
    for (Map.Entry<String, Integer> entry : IdentifierListDelivererDC.getCursorCollector()
        .entrySet()) {
      resToken = entry.getKey();
    }
    System.out.println("HASH MAP BEFORE: ");
    System.out.println(IdentifierListDelivererDC.getCursorCollector());
    System.out.println(resToken);
    String p2 = this.request.getRequest("ListIdentifiers", "", "", "", "", "", resToken);
    System.out.println(p2);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(IdentifierListDelivererDC.getCursorCollector());
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testListIdentifierIDIOMInvalidResumptionToken() throws ParseException {

    System.out
        .println("Test for the verb \"ListIdentifiers\" in IDIOM with an invalid resumption token");
    String p = this.request.getRequest("ListIdentifiers", "", "", "", "", "",
        "45d8b524-b69e-44e8-81a9-a3634005a430");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testMissingVerbArgument() throws ParseException {

    System.out.println("Test for request with missing verb argument");
    String p = this.request.getRequest("", "", "", "", "", "", "");
    System.out.println(p);
    System.out.println("-----------------------------------\n");
  }


  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListIdentifierIDIOMWithResumptionToken() throws ParseException {

    System.out.println("Test for the verb \"ListIdentifiers\" in IDIOM with successfull response");
    String p = this.request.getRequest("ListIdentifiers", "", OaipmhConstants.METADATA_IDIOM_PREFIX,
        "", "", "", "");
    System.out.println(p);
    String resToken = "";
    for (Map.Entry<String, Integer> entry : IdentifierListDelivererIdiom.getCursorCollector()
        .entrySet()) {
      resToken = entry.getKey();
    }
    String p2 = this.request.getRequest("ListIdentifiers", "", "", "", "", "", resToken);
    System.out.println(p2);
    System.out.println("HASH MAP AFTER: ");
    System.out.println(IdentifierListDelivererIdiom.getCursorCollector());
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecordSets() throws ParseException {

    TestTGOaipmhLocally.recordList.setContributors(OaipmhTGConstants.CONTRIBUTOR_LIST);
    TestTGOaipmhLocally.recordList.setCoverages(OaipmhTGConstants.COVERAGE_LIST);
    TestTGOaipmhLocally.recordList.setCreators(OaipmhTGConstants.CREATOR_LIST);
    TestTGOaipmhLocally.recordList.setDates(OaipmhTGConstants.DATE_LIST);
    TestTGOaipmhLocally.recordList.setDescriptions(OaipmhTGConstants.DESCRIPTION_LIST);
    TestTGOaipmhLocally.recordList.setFormats(OaipmhTGConstants.FORMAT_LIST);
    TestTGOaipmhLocally.recordList.setIdentifiers(OaipmhTGConstants.IDENTIFIER_LIST);
    TestTGOaipmhLocally.recordList.setLanguages(OaipmhTGConstants.LANGUAGE_LIST);
    TestTGOaipmhLocally.recordList.setPublishers(OaipmhTGConstants.PUBLISHER_LIST);
    TestTGOaipmhLocally.recordList.setRelations(OaipmhTGConstants.RELATIONS_LIST);
    TestTGOaipmhLocally.recordList
        .setRelationsForWork(OaipmhTGConstants.RELATIONS_FOR_WORK_LIST);
    TestTGOaipmhLocally.recordList.setRights(OaipmhTGConstants.RIGHTS_LIST);
    TestTGOaipmhLocally.recordList.setSources(OaipmhTGConstants.SOURCE_LIST);
    TestTGOaipmhLocally.recordList.setSubjects(OaipmhTGConstants.SUBJECT_LIST);
    TestTGOaipmhLocally.recordList.setTitles(OaipmhTGConstants.TITLE_LIST);
    TestTGOaipmhLocally.recordList.setTypes(OaipmhTGConstants.TYPE_LIST);
    TestTGOaipmhLocally.recordList.setFormatField(OaipmhTGConstants.FORMAT);
    TestTGOaipmhLocally.recordList.setFormatToFilter(TextGridMimetypes.EDITION);
    TestTGOaipmhLocally.recordList.setDateOfObjectCreation(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.recordList
        .setRelationToFurtherMetadataObject(OaipmhTGConstants.EDITION_ISEDITIONOF);
    TestTGOaipmhLocally.recordList
        .setRepositoryObjectURIPrefix(OaipmhTGConstants.ITEM_IDENTIFIER_PREFIX);
    TestTGOaipmhLocally.recordList.setRangeField(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.recordList.setModifiedField(OaipmhTGConstants.LAST_MODIFIED);
    TestTGOaipmhLocally.recordList.setIdentifierField("textgridUri");

    System.out.println("Test for the verb \"ListRecords\" with sets with successful response");
    // String p = this.request.getRequest("ListRecords", "", "oai_dc",
    // "project:TGPR-f89ad029-4eb2-ae5c-6028-5db876513128", "", "", "");
    String p = this.request.getRequest("ListRecords", "", "oai_dc", "", "", "", "");

    System.out.println(p);
    /*
     * String resToken = ""; for (Map.Entry<String, Integer> entry :
     * RecordListDelivererDC.cursorCollector.entrySet()) { resToken = entry.getKey(); String p2 =
     * this.request.getRequest("ListRecords", "", "", "", "", "", resToken); System.out.println(p2);
     * System.out.println("HASH MAP AFTER: ");
     * System.out.println(RecordListDelivererDC.cursorCollector);
     * System.out.println("-----------------------------------\n"); }
     */
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecordWithResumptionTokenForIDIOM() throws ParseException {

    System.out.println("Test for the verb \"ListRecords\" for IDIOM with successful response");
    String p = this.request.getRequest("ListRecords", "", OaipmhConstants.METADATA_IDIOM_PREFIX, "",
        "", "", "");
    System.out.println(p);
    String resToken = "";
    for (Map.Entry<String, Integer> entry : RecordListDelivererIdiom.getCursorCollector()
        .entrySet()) {
      resToken = entry.getKey();
      String p2 = this.request.getRequest("ListRecords", "", "", "", "", "", resToken);
      System.out.println(p2);
      System.out.println("HASH MAP AFTER: ");
      System.out.println(RecordListDelivererIdiom.getCursorCollector());
      System.out.println("-----------------------------------\n");
    }
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecords() throws ParseException {

    String r =
        this.request.getRequest("ListRecords", "", "oai_dc", "", "2011-02-06T20:48:39", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListRecords2() throws ParseException {

    TestTGOaipmhLocally.recordList.setContributors(OaipmhTGConstants.CONTRIBUTOR_LIST);
    TestTGOaipmhLocally.recordList.setCoverages(OaipmhTGConstants.COVERAGE_LIST);
    TestTGOaipmhLocally.recordList.setCreators(OaipmhTGConstants.CREATOR_LIST);
    TestTGOaipmhLocally.recordList.setDates(OaipmhTGConstants.DATE_LIST);
    TestTGOaipmhLocally.recordList.setDescriptions(OaipmhTGConstants.DESCRIPTION_LIST);
    TestTGOaipmhLocally.recordList.setFormats(OaipmhTGConstants.FORMAT_LIST);
    TestTGOaipmhLocally.recordList.setIdentifiers(OaipmhTGConstants.IDENTIFIER_LIST);
    TestTGOaipmhLocally.recordList.setLanguages(OaipmhTGConstants.LANGUAGE_LIST);
    TestTGOaipmhLocally.recordList.setPublishers(OaipmhTGConstants.PUBLISHER_LIST);
    TestTGOaipmhLocally.recordList.setRelations(OaipmhTGConstants.RELATIONS_LIST);
    TestTGOaipmhLocally.recordList
        .setRelationsForWork(OaipmhTGConstants.RELATIONS_FOR_WORK_LIST);
    TestTGOaipmhLocally.recordList.setRights(OaipmhTGConstants.RIGHTS_LIST);
    TestTGOaipmhLocally.recordList.setSources(OaipmhTGConstants.SOURCE_LIST);
    TestTGOaipmhLocally.recordList.setSubjects(OaipmhTGConstants.SUBJECT_LIST);
    TestTGOaipmhLocally.recordList.setTitles(OaipmhTGConstants.TITLE_LIST);
    TestTGOaipmhLocally.recordList.setTypes(OaipmhTGConstants.TYPE_LIST);
    TestTGOaipmhLocally.recordList.setFormatField(OaipmhTGConstants.FORMAT);
    TestTGOaipmhLocally.recordList.setFormatToFilter(TextGridMimetypes.EDITION);
    TestTGOaipmhLocally.recordList.setDateOfObjectCreation(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.recordList
        .setRelationToFurtherMetadataObject(OaipmhTGConstants.EDITION_ISEDITIONOF);
    TestTGOaipmhLocally.recordList
        .setRepositoryObjectURIPrefix(OaipmhTGConstants.ITEM_IDENTIFIER_PREFIX);
    TestTGOaipmhLocally.recordList.setRangeField(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.recordList.setModifiedField(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.recordList.setIdentifierField(OaipmhTGConstants.URI);
    TestTGOaipmhLocally.recordList.setSearchResponseSize(100);
    String r = this.request.getRequest("ListRecords", "", "oai_dc", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListMetadataFormatsForIdentifier() throws ParseException {

    String r =
        this.request.getRequest("ListMetadataFormats", "textgrid:125x1.0", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testListMetadataFormatsWithoutID() throws ParseException {

    String r = this.request.getRequest("ListMetadataFormats", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  @Ignore
  public void testListSetsTG() throws ParseException {

    TestTGOaipmhLocally.setListTextGrid.setFormatField("format");
    TestTGOaipmhLocally.setListTextGrid.setFormatToFilter(TextGridMimetypes.EDITION);
    TestTGOaipmhLocally.setListTextGrid.setIdentifierField("textgridUri");
    TestTGOaipmhLocally.setListTextGrid.setRepositoryObjectURIPrefix("textgrid:");
    String r = this.request.getRequest("ListSets", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testEmptyRequest() throws ParseException {

    TestTGOaipmhLocally.identifierList.setFieldForRange(OaipmhTGConstants.RANGE_FIELD);
    TestTGOaipmhLocally.identifierList
        .setIdentifierListFields(OaipmhTGConstants.IDENTIFIER_LIST_FIELDS);
    TestTGOaipmhLocally.identifierList.setDateOfObjectCreation(OaipmhTGConstants.CREATED);
    TestTGOaipmhLocally.identifierList.setIdentifierField("textgridUri");
    TestTGOaipmhLocally.identifierList.setRepositoryObjectURIPrefix("textgrid:");
    TestTGOaipmhLocally.identifierList.setSearchResponseSize("100");
    String r = this.request.getRequest("", "", "", "", "", "", "");
    System.out.println(r);
    System.out.println("-----------------------------------\n");
  }

}
